@@ -270,7 +270,8 @@ _is_expired(gpointer key, gpointer value, gpointer data)
 {
 	NotifyTimeout *nt = (NotifyTimeout *)value;
 	gboolean *phas_more_timeouts = (gboolean *)data;
-	glong remaining;
+	time_t now_time;
+	time_t expiration_time;
 	GTimeVal now;
 
 	if (!nt->has_timeout)
@@ -278,17 +279,18 @@ _is_expired(gpointer key, gpointer value, gpointer data)
 
 	g_get_current_time(&now);
 
-	remaining = ((nt->expiration.tv_sec * 1000) +
-				 (nt->expiration.tv_usec / 1000)) -
-	            ((now.tv_sec * 1000) + (now.tv_usec / 1000));
+	expiration_time = (nt->expiration.tv_sec * 1000) +
+	                  (nt->expiration.tv_usec / 1000);
+	now_time = (now.tv_sec * 1000) + (now.tv_usec / 1000);
 
-	theme_notification_tick(nt->nw, remaining);
-
-	if (remaining <= 0)
+	if (now_time > expiration_time)
 	{
+		theme_notification_tick(nt->nw, 0);
 		_emit_closed_signal(G_OBJECT(nt->nw));
 		return TRUE;
 	}
+	else
+		theme_notification_tick(nt->nw, expiration_time - now_time);
 
 	*phas_more_timeouts = TRUE;
 
@@ -317,7 +319,7 @@ _calculate_timeout(NotifyDaemon *daemon, NotifyTimeout *nt, int timeout)
 		nt->has_timeout = FALSE;
 	else
 	{
-		gulong usec;
+		glong usec;
 
 		nt->has_timeout = TRUE;
 
@@ -327,6 +329,15 @@ _calculate_timeout(NotifyDaemon *daemon, NotifyTimeout *nt, int timeout)
 		theme_set_notification_timeout(nt->nw, timeout);
 
 		usec = timeout * 1000;	/* convert from msec to usec */
+
+		/*
+		 * If it's less than 0, wrap around back to MAXLONG.
+		 * g_time_val_add() requires a glong, and anything larger than
+		 * MAXLONG will be treated as a negative value.
+		 */
+		if (usec < 0)
+			usec = G_MAXLONG;
+
 		g_get_current_time(&nt->expiration);
 		g_time_val_add(&nt->expiration, usec);
 
@@ -853,6 +864,10 @@ notify_daemon_notify_handler(NotifyDaemon *daemon,
 		g_signal_connect(G_OBJECT(nw), "destroy",
 						 G_CALLBACK(_notification_destroyed_cb), daemon);
 	}
+	else
+	{
+		theme_clear_notification_actions(nw);
+	}
 
 	theme_set_notification_text(nw, summary, body);
 	theme_set_notification_hints(nw, hints);
@@ -894,23 +909,6 @@ notify_daemon_notify_handler(NotifyDaemon *daemon,
 		}
 	}
 
-	if (use_pos_data)
-	{
-		/*
-		 * Typically, the theme engine will set its own position based on
-		 * the arrow X, Y hints. However, in case, move the notification to
-		 * that position.
-		 */
-		theme_set_notification_arrow(nw, TRUE, x, y);
-		theme_move_notification(nw, x, y);
-	}
-	else
-	{
-		theme_set_notification_arrow(nw, FALSE, 0, 0);
-		_notify_daemon_add_bubble_to_poptart_stack(daemon, nw,
-												   new_notification);
-	}
-
 	/* check for icon_data if icon == "" */
 	if (*icon == '\0')
 	{
@@ -934,7 +932,7 @@ notify_daemon_notify_handler(NotifyDaemon *daemon,
 		else
 		{
 			/* Load icon theme icon */
-			GtkIconTheme *theme = gtk_icon_theme_new();
+			GtkIconTheme *theme = gtk_icon_theme_get_default();
 			GtkIconInfo *icon_info =
 				gtk_icon_theme_lookup_icon(theme, icon, IMAGE_SIZE,
 										   GTK_ICON_LOOKUP_USE_BUILTIN);
@@ -943,7 +941,7 @@ notify_daemon_notify_handler(NotifyDaemon *daemon,
 			{
 				pixbuf = gtk_icon_theme_load_icon(
 					theme, icon,
-					MAX(IMAGE_SIZE, gtk_icon_info_get_base_size(icon_info)),
+					MIN(IMAGE_SIZE, gtk_icon_info_get_base_size(icon_info)),
 					GTK_ICON_LOOKUP_USE_BUILTIN, NULL);
 
 				gtk_icon_info_free(icon_info);
@@ -963,6 +961,23 @@ notify_daemon_notify_handler(NotifyDaemon *daemon,
 		}
 	}
 
+	if (use_pos_data)
+	{
+		/*
+		 * Typically, the theme engine will set its own position based on
+		 * the arrow X, Y hints. However, in case, move the notification to
+		 * that position.
+		 */
+		theme_set_notification_arrow(nw, TRUE, x, y);
+		theme_move_notification(nw, x, y);
+	}
+	else
+	{
+		theme_set_notification_arrow(nw, FALSE, 0, 0);
+		_notify_daemon_add_bubble_to_poptart_stack(daemon, nw,
+												   new_notification);
+	}
+
 	if (!screensaver_active(GTK_WIDGET(nw)) &&
 		!fullscreen_window_exists(GTK_WIDGET(nw)))
 	{
